import os


def safe_open(filename, mode='r'):
    if 'w' in mode:
        dirname = os.path.dirname(filename)
        if dirname and not os.path.exists(dirname):
            os.makedirs(dirname)

    if filename.endswith('.gz'):
        import gzip
        return gzip.open(filename, mode=mode)

    return open(filename, mode=mode)





import json


class Article(object):
    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)

    def to_dict(self):
        return self.__dict__

    def to_json(self, **kwargs):
        return json.dumps(self.__dict__, ensure_ascii=False, **kwargs)

    @property
    def fields(self):
        return list(self.__dict__.keys())

    def __repr__(self):
        return 'Article[{pmid} - {title}]'.format(**self.__dict__)


if __name__ == '__main__':
    
    p = Article(pmid=1, issn='1234-5678', title='test')
    print(p)
    print(p.to_dict())
    print(p.to_json())


import os
import re
import click
import datetime

from dateutil.parser import parse as date_parse

from pypubmed.core.export import Export

search_examples = click.style('''
examples:

    pypubmed search ngs -l 5 -o ngs.xlsx

    pypubmed search 'NGS[Title] AND Disease[Title/Abstract]' -o ngs_disease.xlsx

    pypubmed search 1,2,3,4

    pypubmed search pmid_list.txt
''', fg='yellow')
@click.command(help=click.style('search with pmid or a term', bold=True, fg='green'), epilog=search_examples, no_args_is_help=True)
@click.option('-c', '--cited', help='get cited information', default=False, is_flag=True)
@click.option('-n', '--no-translate', help='do not translate the abstract', default=False, is_flag=True)
@click.option('-b', '--batch-size', help='the batch size for efetch', default=10, type=int, show_default=True)
@click.option('-min', '--min-factor', help='filter with IF', type=float)
@click.option('-l', '--limit', help='limit the count of output', type=int)
@click.option('-f', '--fields', help='the fields to export')
@click.option('-o', '--outfile', help='the output filename', default='pubmed.xlsx', show_default=True)
@click.argument('term', nargs=1)
@click.pass_obj
def search(obj, **kwargs):
    
    articles = obj['eutils'].search(translate=not kwargs['no_translate'], **kwargs)

    data = []
    for n, article in enumerate(articles, 1):
        obj['eutils'].logger.debug('{}. {}'.format(n, article))

        # filter impact factor
        if kwargs['min_factor'] and (article.impact_factor != '.' and article.impact_factor < kwargs['min_factor']):
            continue

        data.append(article.to_dict())
        if kwargs['limit'] and n >= kwargs['limit']:
            break

    if kwargs['min_factor']:
        obj['eutils'].logger.info('A total of {} articles were found, {} remaining after filtering IF>={}'.format(n, len(data), kwargs['min_factor']))

    Export(data, **kwargs).export()


@click.command(help=click.style('generate advance search string', bold=True, fg='cyan'))
@click.pass_obj
def advance_search(obj, **kwargs):
    fields = obj['eutils'].validate_fields()

    query_box = ''
    while True:
        number = click.prompt('>>> please choose a number of field', type=int, default=48)
        field = fields[number][1]
        click.secho('your choice is: {number} - {field}'.format(**locals()), fg='cyan')

        if field.startswith('Date - '):
            start = click.prompt('>>> please enter the start date(YYYY-MM-DD)', value_proc=date_parse).strftime('%Y/%m/%d')
            end = click.prompt('>>> please enter the end date', default='3000', value_proc=date_parse)
            if end != '3000':
                end = end.strftime('%Y/%m/%d')
            term = '("{start}"[{field}] : "{end}"[{field}])'.format(**locals())
        else:
            term = click.prompt('>>> please enter a search term')
            if field == 'All Fields':
                term = '{term}'.format(**locals())
            else:
                term = '"{term}"[{field}]'.format(**locals())

        if not query_box:
            query_box = term
        else:
            logic = click.prompt('>>> please input the logic', type=click.Choice(['and', 'or', 'not']), default='and').upper()
            query_box = '({query_box}) {logic} ({term})'.format(**locals())

        click.secho('query box now: ' + query_box, fg='bright_green')

        if click.confirm('input finish?'):
            break

    click.secho('final query box: {}'.format(query_box), fg='bright_cyan')
    res = obj['eutils'].esearch(query_box, retmax=1, head=True)

    if res['count']:
        details = []
        for each in res['translationstack']:
            if isinstance(each, dict):
                details += ['{term}:{count}'.format(**each)]
        click.secho('count:\t{count}\nquery:\t{querytranslation}\ndetail:\t{detail}'.format(detail=', '.join(details), **res), fg='yellow')

        if click.confirm('search with this query box?'):
            outfile = re.sub(r'[\'"\(\)\[\]/ ]+', '_', query_box).strip('_') + '.xlsx'
            outfile = click.prompt('output filename', default=outfile, show_default=True)
            os.system('pypubmed search "{}" -o {}'.format(query_box, outfile))
    else:
        click.echo(res)



# -*- coding=utf-8 -*-
import os
import re
import datetime

try:
    import lxml.etree as ET
except ImportError:
    import xml.etree.cElementTree as ET

from w3lib import html


SPECIAL_CHARS = {
    u'\u2009': ' ',
    u'\u202f': ' ',
    u'\u2217': '*',
}


def parse_abstract(AbstractTexts):
    """
        - 1 no AbstractText
        - 2 sigle AbstractText
        - 3 multiple AbstractTexts with Labels
        
        * remove tags
        * repalce entities
        * repalce special chars
    """
    if not AbstractTexts:
        abstract = '.'
    elif len(AbstractTexts) == 1:
        abstract = ''.join(AbstractTexts[0].itertext()) or '.'
    else:
        abstracts = []
        for each in AbstractTexts:
            label = each.attrib.get('Label')
            text = ''.join(each.itertext())
            if label:
                abstracts.append('{}: {}'.format(label, text))
            else:
                abstracts.append(text)
        abstract = '\n'.join(abstracts)

    # ===========
    # remove tags
    # ===========
    abstract = html.remove_tags(abstract)

    # ===============
    # repalce entities
    # ===============
    n = 0
    while re.search(r'&.{1,7};', abstract):
        abstract = html.replace_entities(abstract)
        n += 1
        if n > 4:
            print('entities levels > 5')
            print(re.findall(r'&.{1,7};', abstract))
            break

    # =====================
    # replace special chars
    # =====================
    for special, replace in SPECIAL_CHARS.items():
        abstract = abstract.replace(special, replace)

    return abstract


def parse(xml):

    if os.path.isfile(xml):
        tree = ET.parse(xml)
    else:
        tree = ET.fromstring(xml)
    
    if tree.find('PubmedArticle') is None:
        yield None
    else:
        for PubmedArticle in tree.iterfind('PubmedArticle'):
            context = {}
            MedlineCitation = PubmedArticle.find('MedlineCitation')
            Article = MedlineCitation.find('Article')

            context['pmid'] = int(MedlineCitation.findtext('PMID'))

            context['e_issn'] = Article.findtext('Journal/ISSN[@IssnType="Electronic"]')
            context['issn'] = Article.findtext('Journal/ISSN[@IssnType="Print"]') or MedlineCitation.findtext('MedlineJournalInfo/ISSNLinking')

            context['journal'] = Article.findtext('Journal/Title')
            context['iso_abbr'] = Article.findtext('Journal/ISOAbbreviation')

            context['med_abbr'] = MedlineCitation.findtext('MedlineJournalInfo/MedlineTA')

            context['pubdate'] = ' '.join(Article.xpath('Journal/JournalIssue/PubDate/*/text()'))

            pubmed_pubdate = year = ''
            for status in ('pubmed', 'entrez', 'medline'):
                ymd = PubmedArticle.xpath('PubmedData/History/PubMedPubDate[@PubStatus="{}"]/*/text()'.format(status))
                if ymd:
                    pubmed_pubdate = datetime.datetime(*map(int, ymd))
                    year = pubmed_pubdate.year
                    pubmed_pubdate = pubmed_pubdate.strftime('%Y/%m/%d')
                    break

            context['year'] = year
            context['pubmed_pubdate'] = pubmed_pubdate

            context['pagination'] = Article.findtext('Pagination/MedlinePgn')
            context['volume'] = Article.findtext('Journal/JournalIssue/Volume')
            context['issue'] = Article.findtext('Journal/JournalIssue/Issue')
            context['title'] = ''.join(Article.find('ArticleTitle').itertext())
            context['keywords'] = MedlineCitation.xpath('KeywordList/Keyword/text()')
            context['pub_status'] = PubmedArticle.findtext('PubmedData/PublicationStatus')

            context['abstract'] = parse_abstract(Article.xpath('Abstract/AbstractText'))
            
            author_list = []
            for author in Article.xpath('AuthorList/Author'):
                name = [each.text for each in author.xpath('*')][:3]  # LastName, ForeName, Initials
                author_list.append(name)
            context['authors'] = author_list

            context['pub_types'] = Article.xpath('PublicationTypeList/PublicationType/text()')
            context['doi'] = PubmedArticle.findtext('PubmedData/ArticleIdList/ArticleId[@IdType="doi"]')
            context['pmc'] = PubmedArticle.findtext('PubmedData/ArticleIdList/ArticleId[@IdType="pmc"]')

            yield context


if __name__ == '__main__':
    import json
    from webrequests import WebRequest
    
    url = 'https://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?db=pubmed&id=17284678,9997&retmode=xml'
    resp = WebRequest.get_response(url)
    for context in parse(resp.text):
        print(json.dumps(context, indent=2))
